"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.prettifyReadme = exports.prettifyPackageDistJson = exports.prettifyPackageJson = exports.prettifySrc = void 0;
const fs_extra_1 = __importDefault(require("fs-extra"));
const klaw_1 = __importDefault(require("klaw"));
const path_1 = __importDefault(require("path"));
const prettier_1 = __importDefault(require("prettier"));
/**
 * @param basePath -
 * @param srcPath -
 * @param ci -
 * @returns true if the prettify src process was successful
 */
async function prettifySrc(basePath, srcPath, ci) {
    console.log("Prettier - started on src");
    let res;
    try {
        for await (const file of (0, klaw_1.default)(srcPath)) {
            if (file.stats.isDirectory()) {
                continue;
            }
            const contents = await fs_extra_1.default.readFile(file.path, "utf8"), options = (await prettier_1.default.resolveConfig(basePath)) ?? {};
            options.printWidth = 120;
            options.endOfLine = "lf";
            options.parser = "typescript";
            options.tabWidth = 4;
            options.arrowParens = "avoid";
            if (ci) {
                if (!(await prettier_1.default.check(contents, options))) {
                    throw new Error(`${file.path} is not formatted correctly`);
                }
            }
            else {
                const formatted = await prettier_1.default.format(contents, options);
                await fs_extra_1.default.writeFile(file.path, formatted, "utf8");
            }
        }
        res = true;
    }
    catch (e) {
        console.error(e);
        res = false;
    }
    console.log("Prettier - done on src");
    return res;
}
exports.prettifySrc = prettifySrc;
/**
 * @param basePath -
 * @param ci -
 * @returns true if the prettify package.json process was successful
 */
async function prettifyPackageJson(basePath, ci) {
    console.log("Prettier - started on package.json");
    let res;
    try {
        const contents = await fs_extra_1.default.readFile("package.json", "utf8"), options = (await prettier_1.default.resolveConfig(basePath)) ?? {};
        options.tabWidth = 2;
        options.printWidth = 120;
        options.endOfLine = "lf";
        options.parser = "json";
        if (ci) {
            if (!(await prettier_1.default.check(contents, options))) {
                throw new Error(`package.json is not formatted correctly`);
            }
        }
        else {
            const formatted = await prettier_1.default.format(contents, options);
            await fs_extra_1.default.writeFile("package.json", formatted, "utf8");
        }
        res = true;
    }
    catch (e) {
        console.error(e);
        res = false;
    }
    console.log("Prettier - done on package.json");
    return res;
}
exports.prettifyPackageJson = prettifyPackageJson;
/**
 * @param basePath -
 * @param _ci -
 * @returns true if the prettify package.dist.json process was successful
 */
async function prettifyPackageDistJson(basePath, _ci) {
    console.log("Prettier - started on package.dist.json");
    let res;
    try {
        const contents = await fs_extra_1.default.readFile("package.dist.json", "utf8"), options = (await prettier_1.default.resolveConfig(basePath)) ?? {};
        options.tabWidth = 2;
        options.printWidth = 120;
        options.endOfLine = "lf";
        options.parser = "json";
        // TODO: disabled this check until "prettier-plugin-multiline-arrays" package is compatible with Prettier 3.0.0
        /* if (ci) {
            if (!(await prettier.check(contents, options))) {
                throw new Error(`package.dist.json is not formatted correctly`);
            }
        } else { */
        const formatted = await prettier_1.default.format(contents, options);
        await fs_extra_1.default.writeFile("package.dist.json", formatted, "utf8");
        // }
        res = true;
    }
    catch (e) {
        console.error(e);
        res = false;
    }
    console.log("Prettier - done on package.dist.json");
    return res;
}
exports.prettifyPackageDistJson = prettifyPackageDistJson;
/**
 * @param basePath -
 * @param ci -
 * @returns true if the prettify readme process was successful
 */
async function prettifyReadme(basePath, ci) {
    console.log("Prettier - started on README.md");
    let res;
    try {
        const contents = await fs_extra_1.default.readFile("README.md", "utf8"), options = (await prettier_1.default.resolveConfig(basePath)) ?? {};
        options.printWidth = 120;
        options.endOfLine = "lf";
        options.parser = "markdown";
        if (ci) {
            if (!(await prettier_1.default.check(contents, options))) {
                throw new Error(`README.md is not formatted correctly`);
            }
        }
        else {
            const formatted = await prettier_1.default.format(contents, options);
            await fs_extra_1.default.writeFile("README.md", formatted, "utf8");
        }
        res = (await prettifyTraductions(basePath, ci)) && (await prettifyMarkdownTypeDocFiles(basePath, ci));
    }
    catch (e) {
        console.error(e);
        res = false;
    }
    console.log("Prettier - done on README.md");
    return res;
}
exports.prettifyReadme = prettifyReadme;
/**
 * @param basePath -
 * @param ci -
 * @returns true if the prettify traductions process was successful
 */
async function prettifyTraductions(basePath, ci) {
    console.log("Prettier - started on traductions");
    let res = false;
    try {
        const folder = "traduction", folderPath = path_1.default.join(basePath, folder);
        if (!fs_extra_1.default.existsSync(folderPath)) {
            res = true;
        }
        if (!res) {
            for await (const file of (0, klaw_1.default)(folderPath)) {
                if (file.stats.isDirectory()) {
                    continue;
                }
                const contents = await fs_extra_1.default.readFile(file.path, "utf8"), options = (await prettier_1.default.resolveConfig(basePath)) ?? {};
                options.printWidth = 120;
                options.endOfLine = "lf";
                options.parser = "markdown";
                if (ci) {
                    if (!(await prettier_1.default.check(contents, options))) {
                        throw new Error(`${file.path} is not formatted correctly`);
                    }
                }
                else {
                    const formatted = await prettier_1.default.format(contents, options);
                    await fs_extra_1.default.writeFile(file.path, formatted, "utf8");
                }
            }
            res = true;
        }
    }
    catch (e) {
        console.error(e);
        res = false;
    }
    console.log("Prettier - done on traductions");
    return res;
}
/**
 * @param basePath -
 * @param ci -
 * @returns true if the prettify markdown typedoc files process was successful
 */
async function prettifyMarkdownTypeDocFiles(basePath, ci) {
    console.log("Prettier - started on markdown");
    let res = false;
    try {
        const folder = "markdown", folderPath = path_1.default.join(basePath, folder);
        if (!fs_extra_1.default.existsSync(folderPath)) {
            res = true;
        }
        if (!res) {
            for await (const file of (0, klaw_1.default)(folderPath)) {
                if (file.stats.isDirectory()) {
                    continue;
                }
                const contents = await fs_extra_1.default.readFile(file.path, "utf8"), options = (await prettier_1.default.resolveConfig(basePath)) ?? {};
                options.printWidth = 120;
                options.endOfLine = "lf";
                options.parser = "markdown";
                if (ci) {
                    if (!(await prettier_1.default.check(contents, options))) {
                        throw new Error(`${file.path} is not formatted correctly`);
                    }
                }
                else {
                    const formatted = await prettier_1.default.format(contents, options);
                    await fs_extra_1.default.writeFile(file.path, formatted, "utf8");
                }
            }
            res = true;
        }
    }
    catch (e) {
        console.error(e);
        res = false;
    }
    console.log("Prettier - done on markdown");
    return res;
}
