"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.createPluginTemplate = void 0;
const string_utils_1 = require("../../utils/string-utils");
const template_utils_1 = require("../../utils/template-utils");
const fs_extra_1 = __importDefault(require("fs-extra"));
const path_1 = __importDefault(require("path"));
const file_utils_1 = require("../../utils/file-utils");
/**
 * Updates the index file with the correct function name
 * @param destPath - The path where the project is located
 * @param name - The name of the project
 */
async function updateIndexFile(destPath, name) {
    const capitalizedName = (0, string_utils_1.capitalize)(name, "-", " "), camelizedName = (0, string_utils_1.camelize)(capitalizedName);
    await (0, file_utils_1.replaceTokensInFile)({
        path: path_1.default.resolve(destPath, "src", "index.ts"),
        tokens: [
            {
                from: /loadTemplatePlugin/g,
                to: `load${capitalizedName}Plugin`,
            },
            {
                from: /"#template#"/g,
                to: `"${camelizedName}"`,
            },
        ],
    });
}
/**
 * Updates the plugin package file
 * @param destPath - The path where the project is located
 * @param name - The name of the project
 * @param description - The description of the project
 * @param repoUrl - The repository url
 */
async function updatePluginPackageFile(destPath, name, description, repoUrl) {
    const camelizedName = (0, string_utils_1.camelize)((0, string_utils_1.camelize)(name, "-"), " "), dashedName = (0, string_utils_1.dash)(camelizedName);
    await (0, template_utils_1.updatePackageFile)(destPath, `tsparticles-plugin-${dashedName}`, description, `tsparticles.plugin.${camelizedName}.min.js`, repoUrl);
}
/**
 * Updates the plugin package dist file
 * @param destPath - The path where the project is located
 * @param name - The name of the project
 * @param description - The description of the project
 * @param repoUrl - The repository url
 */
async function updatePluginPackageDistFile(destPath, name, description, repoUrl) {
    const camelizedName = (0, string_utils_1.camelize)((0, string_utils_1.camelize)(name, "-"), " "), dashedName = (0, string_utils_1.dash)(camelizedName);
    await (0, template_utils_1.updatePackageDistFile)(destPath, `tsparticles-plugin-${dashedName}`, description, `tsparticles.plugin.${camelizedName}.min.js`, repoUrl);
}
/**
 * Updates the plugin readme file
 * @param destPath - The path where the project is located
 * @param name - The name of the project
 * @param description - The description of the project
 * @param repoUrl - The repository url
 */
async function updateReadmeFile(destPath, name, description, repoUrl) {
    const readmePath = path_1.default.resolve(destPath, "README.md"), capitalizedName = (0, string_utils_1.capitalize)(name, "-", " "), camelizedName = (0, string_utils_1.camelize)(capitalizedName), dashedName = (0, string_utils_1.dash)(camelizedName), stringSearch = "github.com", trailingSlashSearch = "github.com/", repoPath = repoUrl.includes(stringSearch)
        ? repoUrl.substring(repoUrl.indexOf(trailingSlashSearch) + trailingSlashSearch.length, repoUrl.indexOf(".git"))
        : "tsparticles/plugin-template";
    await (0, file_utils_1.replaceTokensInFile)({
        path: readmePath,
        tokens: [
            {
                from: /tsParticles Template Plugin/g,
                to: `tsParticles ${description} Plugin`,
            },
            {
                from: /tsparticles-plugin-template/g,
                to: `tsparticles-plugin-${dashedName}`,
            },
            {
                from: /tsparticles\.plugin\.template(\.bundle)?\.min\.js/g,
                to: `tsparticles.plugin.${camelizedName}$1.min.js`,
            },
            {
                from: /loadTemplatePlugin/g,
                to: `load${capitalizedName}Plugin`,
            },
            {
                from: /\[tsParticles]\(https:\/\/github.com\/matteobruni\/tsparticles\) additional template plugin\./g,
                to: `[tsParticles](https://github.com/matteobruni/tsparticles) additional ${name} plugin.`,
            },
            {
                from: /plugin\.type: "template"/g,
                to: `plugin.type: "${camelizedName}"`,
            },
            {
                from: /!\[demo]\(https:\/\/raw.githubusercontent.com\/tsparticles\/plugin-template\/main\/images\/sample.png\)/g,
                to: `![demo](https://raw.githubusercontent.com/${repoPath}/main/images/sample.png)`,
            },
        ],
    });
}
/**
 * Updates the plugin webpack file
 * @param destPath - The path where the project is located
 * @param name - The name of the project
 * @param description - The description of the project
 */
async function updatePluginWebpackFile(destPath, name, description) {
    await (0, template_utils_1.updateWebpackFile)(destPath, (0, string_utils_1.camelize)((0, string_utils_1.capitalize)(name, "-", " ")), description, "loadParticlesPlugin");
}
/**
 * Creates the plugin project
 * @param name - The name of the project
 * @param description - The description of the project
 * @param repoUrl - The repository url
 * @param destPath - The path where the project is located
 */
async function createPluginTemplate(name, description, repoUrl, destPath) {
    const sourcePath = path_1.default.resolve(__dirname, "..", "..", "..", "files", "create-plugin");
    await (0, template_utils_1.copyEmptyTemplateFiles)(destPath);
    await fs_extra_1.default.copy(sourcePath, destPath, {
        overwrite: true,
        filter: template_utils_1.copyFilter,
    });
    await updateIndexFile(destPath, name);
    await updatePluginPackageFile(destPath, name, description, repoUrl);
    await updatePluginPackageDistFile(destPath, name, description, repoUrl);
    await updateReadmeFile(destPath, name, description, repoUrl);
    await updatePluginWebpackFile(destPath, name, description);
    await (0, template_utils_1.runInstall)(destPath);
    await (0, template_utils_1.runBuild)(destPath);
}
exports.createPluginTemplate = createPluginTemplate;
