"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.createPresetTemplate = void 0;
const string_utils_1 = require("../../utils/string-utils");
const template_utils_1 = require("../../utils/template-utils");
const fs_extra_1 = __importDefault(require("fs-extra"));
const path_1 = __importDefault(require("path"));
const file_utils_1 = require("../../utils/file-utils");
/**
 * Updates the bundle file with the correct function name
 * @param destPath - The path where the project is located
 * @param name - The name of the project
 */
async function updateBundleFile(destPath, name) {
    const capitalizedName = (0, string_utils_1.capitalize)(name, "-", " ");
    await (0, file_utils_1.replaceTokensInFile)({
        path: path_1.default.resolve(destPath, "src", "bundle.ts"),
        tokens: [
            {
                from: /loadTemplatePreset/g,
                to: `load${capitalizedName}Preset`,
            },
        ],
    });
}
/**
 * Updates the index file with the correct function name
 * @param destPath - The path where the project is located
 * @param name - The name of the project
 */
async function updateIndexFile(destPath, name) {
    const capitalizedName = (0, string_utils_1.capitalize)(name, "-", " "), camelizedName = (0, string_utils_1.camelize)(capitalizedName);
    await (0, file_utils_1.replaceTokensInFile)({
        path: path_1.default.resolve(destPath, "src", "index.ts"),
        tokens: [
            {
                from: /loadTemplatePreset/g,
                to: `load${capitalizedName}Preset`,
            },
            {
                from: /"#template#"/g,
                to: `"${camelizedName}"`,
            },
        ],
    });
}
/**
 * Updates the preset package file
 * @param destPath - The path where the project is located
 * @param name - The name of the project
 * @param description - The description of the project
 * @param repoUrl - The repository url
 */
async function updatePresetPackageFile(destPath, name, description, repoUrl) {
    const camelizedName = (0, string_utils_1.camelize)(name, "-", " "), dashedName = (0, string_utils_1.dash)(camelizedName);
    await (0, template_utils_1.updatePackageFile)(destPath, `tsparticles-preset-${dashedName}`, description, `tsparticles.preset.${camelizedName}.min.js`, repoUrl);
}
/**
 * Updates the preset package dist file
 * @param destPath - The path where the project is located
 * @param name - The name of the project
 * @param description - The description of the project
 * @param repoUrl - The repository url
 */
async function updatePresetPackageDistFile(destPath, name, description, repoUrl) {
    const camelizedName = (0, string_utils_1.camelize)(name, "-", " "), dashedName = (0, string_utils_1.dash)(camelizedName);
    await (0, template_utils_1.updatePackageDistFile)(destPath, `tsparticles-preset-${dashedName}`, description, `tsparticles.preset.${camelizedName}.min.js`, repoUrl);
}
/**
 * Updates the preset readme file
 * @param destPath - The path where the project is located
 * @param name - The name of the project
 * @param description - The description of the project
 * @param repoUrl - The repository url
 */
async function updateReadmeFile(destPath, name, description, repoUrl) {
    const capitalizedName = (0, string_utils_1.capitalize)(name, "-", " "), camelizedName = (0, string_utils_1.camelize)(capitalizedName), dashedName = (0, string_utils_1.dash)(camelizedName), stringSearch = "github.com", trailingSlashSearch = "github.com/", repoPath = repoUrl.includes(stringSearch)
        ? repoUrl.substring(repoUrl.indexOf(trailingSlashSearch) + trailingSlashSearch.length, repoUrl.indexOf(".git"))
        : "tsparticles/preset-template";
    await (0, file_utils_1.replaceTokensInFile)({
        path: path_1.default.resolve(destPath, "README.md"),
        tokens: [
            {
                from: /tsParticles Template Preset/g,
                to: `tsParticles ${description} Preset`,
            },
            {
                from: /tsparticles-preset-template/g,
                to: `tsparticles-preset-${dashedName}`,
            },
            {
                from: /tsparticles\.preset\.template(\.bundle)?\.min\.js/g,
                to: `tsparticles.preset.${camelizedName}$1.min.js`,
            },
            {
                from: /loadTemplatePreset/g,
                to: `load${capitalizedName}Preset`,
            },
            {
                from: /\[tsParticles]\(https:\/\/github.com\/matteobruni\/tsparticles\) preset template\./g,
                to: `[tsParticles](https://github.com/matteobruni/tsparticles) preset ${name}.`,
            },
            {
                from: /preset: "template"/g,
                to: `preset: "${camelizedName}`,
            },
            {
                from: /!\[demo]\(https:\/\/raw.githubusercontent.com\/tsparticles\/preset-template\/main\/images\/sample.png\)/g,
                to: `![demo](https://raw.githubusercontent.com/${repoPath}/main/images/sample.png)`,
            },
        ],
    });
}
/**
 * Updates the preset webpack file
 * @param destPath - The path where the project is located
 * @param name - The name of the project
 * @param description - The description of the project
 */
async function updatePresetWebpackFile(destPath, name, description) {
    await (0, template_utils_1.updateWebpackFile)(destPath, (0, string_utils_1.camelize)((0, string_utils_1.capitalize)(name, "-", " ")), description, "loadParticlesPreset");
}
/**
 * Creates the preset project
 * @param name - The name of the project
 * @param description - The description of the project
 * @param repoUrl - The repository url
 * @param destPath - The path where the project is located
 */
async function createPresetTemplate(name, description, repoUrl, destPath) {
    const sourcePath = path_1.default.resolve(__dirname, "..", "..", "..", "files", "create-preset");
    await (0, template_utils_1.copyEmptyTemplateFiles)(destPath);
    await fs_extra_1.default.copy(sourcePath, destPath, {
        overwrite: true,
        filter: template_utils_1.copyFilter,
    });
    await updateBundleFile(destPath, name);
    await updateIndexFile(destPath, name);
    await updatePresetPackageFile(destPath, name, description, repoUrl);
    await updatePresetPackageDistFile(destPath, name, description, repoUrl);
    await updateReadmeFile(destPath, name, description, repoUrl);
    await updatePresetWebpackFile(destPath, name, description);
    await (0, template_utils_1.runInstall)(destPath);
    await (0, template_utils_1.runBuild)(destPath);
}
exports.createPresetTemplate = createPresetTemplate;
