"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.createShapeTemplate = void 0;
const string_utils_1 = require("../../utils/string-utils");
const template_utils_1 = require("../../utils/template-utils");
const fs_extra_1 = __importDefault(require("fs-extra"));
const path_1 = __importDefault(require("path"));
const file_utils_1 = require("../../utils/file-utils");
/**
 * Updates the index file with the correct function name
 * @param destPath - The path where the project is located
 * @param name - The name of the project
 */
async function updateIndexFile(destPath, name) {
    const capitalizedName = (0, string_utils_1.capitalize)(name, "-", " "), camelizedName = (0, string_utils_1.camelize)(capitalizedName);
    await (0, file_utils_1.replaceTokensInFile)({
        path: path_1.default.resolve(destPath, "src", "index.ts"),
        tokens: [
            {
                from: /loadTemplateShape/g,
                to: `load${capitalizedName}Shape`,
            },
            {
                from: /"#template#"/g,
                to: `"${camelizedName}"`,
            },
        ],
    });
}
/**
 * Updates the shape package file
 * @param destPath - The path where the project is located
 * @param name - The name of the project
 * @param description - The description of the project
 * @param repoUrl - The repository url
 */
async function updateShapePackageFile(destPath, name, description, repoUrl) {
    const camelizedName = (0, string_utils_1.camelize)((0, string_utils_1.camelize)(name, "-"), " "), dashedName = (0, string_utils_1.dash)(camelizedName);
    await (0, template_utils_1.updatePackageFile)(destPath, `tsparticles-shape-${dashedName}`, description, `tsparticles.shape.${camelizedName}.min.js`, repoUrl);
}
/**
 * Updates the shape package dist file
 * @param destPath - The path where the project is located
 * @param name - The name of the project
 * @param description - The description of the project
 * @param repoUrl - The repository url
 */
async function updateShapePackageDistFile(destPath, name, description, repoUrl) {
    const camelizedName = (0, string_utils_1.camelize)((0, string_utils_1.camelize)(name, "-"), " "), dashedName = (0, string_utils_1.dash)(camelizedName);
    await (0, template_utils_1.updatePackageDistFile)(destPath, `tsparticles-shape-${dashedName}`, description, `tsparticles.shape.${camelizedName}.min.js`, repoUrl);
}
/**
 * Updates the shape readme file
 * @param destPath - The path where the project is located
 * @param name - The name of the project
 * @param description - The description of the project
 * @param repoUrl - The repository url
 */
async function updateReadmeFile(destPath, name, description, repoUrl) {
    const capitalizedName = (0, string_utils_1.capitalize)(name, "-", " "), camelizedName = (0, string_utils_1.camelize)(capitalizedName), dashedName = (0, string_utils_1.dash)(camelizedName), stringSearch = "github.com", trailingSlashSearch = "github.com/", repoPath = repoUrl.includes(stringSearch)
        ? repoUrl.substring(repoUrl.indexOf(trailingSlashSearch) + trailingSlashSearch.length, repoUrl.indexOf(".git"))
        : "tsparticles/shape-template";
    await (0, file_utils_1.replaceTokensInFile)({
        path: path_1.default.resolve(destPath, "README.md"),
        tokens: [
            {
                from: /tsParticles Template Shape/g,
                to: `tsParticles ${description} Shape`,
            },
            {
                from: /tsparticles-shape-template/g,
                to: `tsparticles-shape-${dashedName}`,
            },
            {
                from: /tsparticles\.shape\.template(\.bundle)?\.min\.js/g,
                to: `tsparticles.shape.${camelizedName}$1.min.js`,
            },
            {
                from: /loadTemplateShape/g,
                to: `load${capitalizedName}Shape`,
            },
            {
                from: /\[tsParticles]\(https:\/\/github.com\/matteobruni\/tsparticles\) additional template shape\./g,
                to: `[tsParticles](https://github.com/matteobruni/tsparticles) additional ${name} shape.`,
            },
            {
                from: /shape\.type: "template"/g,
                to: `shape.type: "${camelizedName}`,
            },
            {
                from: /!\[demo]\(https:\/\/raw.githubusercontent.com\/tsparticles\/shape-template\/main\/images\/sample.png\)/g,
                to: `![demo](https://raw.githubusercontent.com/${repoPath}/main/images/sample.png)`,
            },
        ],
    });
}
/**
 * Updates the shape webpack file
 * @param destPath - The path where the project is located
 * @param name - The name of the project
 * @param description - The description of the project
 */
async function updateShapeWebpackFile(destPath, name, description) {
    await (0, template_utils_1.updateWebpackFile)(destPath, (0, string_utils_1.camelize)((0, string_utils_1.capitalize)(name, "-", " ")), description, "loadParticlesShape");
}
/**
 * Creates the shape project
 * @param name - The name of the project
 * @param description - The description of the project
 * @param repoUrl - The repository url
 * @param destPath - The path where the project is located
 */
async function createShapeTemplate(name, description, repoUrl, destPath) {
    const sourcePath = path_1.default.resolve(__dirname, "..", "..", "..", "files", "create-shape");
    await (0, template_utils_1.copyEmptyTemplateFiles)(destPath);
    await fs_extra_1.default.copy(sourcePath, destPath, {
        overwrite: true,
        filter: template_utils_1.copyFilter,
    });
    await updateIndexFile(destPath, name);
    await updateShapePackageFile(destPath, name, description, repoUrl);
    await updateShapePackageDistFile(destPath, name, description, repoUrl);
    await updateReadmeFile(destPath, name, description, repoUrl);
    await updateShapeWebpackFile(destPath, name, description);
    await (0, template_utils_1.runInstall)(destPath);
    await (0, template_utils_1.runBuild)(destPath);
}
exports.createShapeTemplate = createShapeTemplate;
