"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.runBuild = exports.runInstall = exports.copyFilter = exports.copyEmptyTemplateFiles = exports.updateWebpackFile = exports.updatePackageDistFile = exports.updatePackageFile = void 0;
const child_process_1 = require("child_process");
const fs_extra_1 = __importDefault(require("fs-extra"));
const lookpath_1 = require("lookpath");
const path_1 = __importDefault(require("path"));
const file_utils_1 = require("./file-utils");
/**
 * Updates the package.json file
 * @param destPath - The path where the package.json file is located
 * @param packageName - The name of the package
 * @param description - The description of the package
 * @param fileName - The name of the output file
 * @param repoUrl - The repository URL
 */
async function updatePackageFile(destPath, packageName, description, fileName, repoUrl) {
    await (0, file_utils_1.replaceTokensInFile)({
        path: path_1.default.resolve(destPath, "package.json"),
        tokens: [
            {
                from: /"tsParticles empty template"/g,
                to: `"${description}"`,
            },
            {
                from: /"tsparticles.empty.template.min.js"/g,
                to: `"${fileName}"`,
            },
            {
                from: /\s{4}"private": true,\r?\n?/g,
                to: "",
            },
            {
                from: /"@tsparticles\/empty-template"/g,
                to: `"${packageName}"`,
            },
            {
                from: /"url": "git\+https:\/\/github\.com\/tsparticles\/empty-template\.git"/g,
                to: `"url": "git+${repoUrl}"`,
            },
            {
                from: /"url": "https:\/\/github\.com\/tsparticles\/empty-template\/issues"/g,
                to: `"url": "${repoUrl.replace(".git", "/issues")}"`,
            },
        ],
    });
}
exports.updatePackageFile = updatePackageFile;
/**
 * Updates the package.dist.json file with the new project name and description
 * @param destPath - The path where the package.dist.json file is located
 * @param packageName - The name of the package
 * @param description - The description of the package
 * @param fileName - The name of the output file
 * @param repoUrl - The url of the repository
 */
async function updatePackageDistFile(destPath, packageName, description, fileName, repoUrl) {
    await (0, file_utils_1.replaceTokensInFile)({
        path: path_1.default.resolve(destPath, "package.dist.json"),
        tokens: [
            {
                from: /"tsParticles empty template"/g,
                to: `"${description}"`,
            },
            {
                from: /"tsparticles.empty.template.min.js"/g,
                to: `"${fileName}"`,
            },
            {
                from: /\s{4}"private": true,\r?\n?/g,
                to: "",
            },
            {
                from: /"@tsparticles\/empty-template"/g,
                to: `"${packageName}"`,
            },
            {
                from: /"url": "git\+https:\/\/github\.com\/tsparticles\/empty-template\.git"/g,
                to: `"url": "git+${repoUrl}"`,
            },
            {
                from: /"url": "https:\/\/github\.com\/tsparticles\/empty-template\/issues"/g,
                to: `"url": "${repoUrl.replace(".git", "/issues")}"`,
            },
        ],
    });
}
exports.updatePackageDistFile = updatePackageDistFile;
/**
 * Updates the webpack file with the new project name and description
 * @param destPath - The path where the project will be created
 * @param name - The name of the project
 * @param description - The description of the project
 * @param fnName - The name of the function to load the template
 */
async function updateWebpackFile(destPath, name, description, fnName) {
    await (0, file_utils_1.replaceTokensInFile)({
        path: path_1.default.resolve(destPath, "webpack.config.js"),
        tokens: [
            {
                from: /"Empty"/g,
                to: `"${description}"`,
            },
            {
                from: /"empty"/g,
                to: `"${name}"`,
            },
            {
                from: /loadParticlesTemplate/g,
                to: fnName,
            },
        ],
    });
}
exports.updateWebpackFile = updateWebpackFile;
/**
 * Copies the empty template files to the destination path
 * @param destPath - The path where the project will be created
 */
async function copyEmptyTemplateFiles(destPath) {
    await fs_extra_1.default.copy(path_1.default.resolve(__dirname, "..", "..", "files", "empty-project"), destPath, {
        overwrite: true,
        filter: copyFilter,
    });
}
exports.copyEmptyTemplateFiles = copyEmptyTemplateFiles;
/**
 * Filters the files to copy
 * @param src - The source file path
 * @returns true if the file should be copied
 */
function copyFilter(src) {
    return !(src.endsWith("node_modules") || src.endsWith("dist"));
}
exports.copyFilter = copyFilter;
/**
 * Runs npm install in the given path
 * @param destPath - The path where the project will be created
 */
async function runInstall(destPath) {
    if (!(await (0, lookpath_1.lookpath)("npm"))) {
        return;
    }
    return new Promise((resolve, reject) => {
        (0, child_process_1.exec)("npm install", {
            cwd: destPath,
        }, error => {
            if (error) {
                reject(error);
                return;
            }
            resolve();
        });
    });
}
exports.runInstall = runInstall;
/**
 * Runs npm run build in the given path
 * @param destPath - The path where the project will be build
 */
async function runBuild(destPath) {
    if (!(await (0, lookpath_1.lookpath)("npm"))) {
        return;
    }
    return new Promise((resolve, reject) => {
        (0, child_process_1.exec)("npm run build", {
            cwd: destPath,
        }, error => {
            if (error) {
                reject(error);
                return;
            }
            resolve();
        });
    });
}
exports.runBuild = runBuild;
